#!/bin/bash

dirname=$(dirname $0)

. ${dirname}/fhgfs_setup.defaults


deploy_meta()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi
   
   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}
   
    line=$(grep "$1" $metaFile)
    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")
    echo "Installing FhGFS Meta Server on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    
    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg -i --force-overwrite"
     check_cmd="dpkg-query -s"
     pack_exten="deb"
     package=$(find ${packdir}/ -name fhgfsmeta_*_${arch}.${pack_exten} | xargs basename)
     packname="fhgfsmeta"
    else
     cmd="rpm -U --replacepkgs"
     check_cmd="rpm -q"
     pack_exten="rpm"
     package=$(find ${packdir}/ -name fhgfs_meta-${dist}*.${arch}.${pack_exten} | xargs basename) 
     packname="fhgfs_meta-${dist}"
    fi
    if [ "$package" = ""  ]
    then
     error="No suitable meta server package found for host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}     
     return 1
    fi
    scp -oStrictHostKeyChecking=no ${packdir}/${package} ${host}:/tmp/ >> ${outFile} 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} /tmp/${package}" >> ${outFile} 2>&1
#    ssh -qq -oStrictHostKeyChecking=no ${host} "${check_cmd} ${packname}" > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to install ${package} on host ${host}"
     echo $error >> ${outFile}
     echo ${host} >> ${failedFile}
     return 1
    fi 
    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_meta.conf.master ${host}:/etc/fhgfs/fhgfs_meta.conf >> $outFile 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "rm /tmp/${package}"
}

deploy_storage()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi
   
   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}

    line=$(grep "$1" $storageFile)
    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")
    echo "Installing FhGFS Storage Server on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg -i --force-overwrite"
     check_cmd="dpkg-query -s"
     pack_exten="deb"
     package=$(find ${packdir}/ -name fhgfsstorage_*_${arch}.${pack_exten} | xargs basename)
     packname="fhgfsstorage"
    else
     cmd="rpm -U --replacepkgs"
     check_cmd="rpm -q"
     pack_exten="rpm"
     package=$(find ${packdir}/ -name fhgfs_storage-${dist}*.${arch}.${pack_exten} | xargs basename)
     packname="fhgfs_storage-${dist}"
    fi
    if [ "$package" = "" ]
    then
     error="No suitable storage server package found for host ${host}"
     echo $error >> $outFile
     echo $host >> ${failedFile}
     return 1
    fi
    scp -oStrictHostKeyChecking=no ${packdir}/${package} ${host}:/tmp/ >> ${outFile} 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} /tmp/${package}" >> ${outFile} 2>&1
    # ssh -qq -oStrictHostKeyChecking=no ${host} "${check_cmd} ${packname}" > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to install ${package} on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_storage.conf.master ${host}:/etc/fhgfs/fhgfs_storage.conf >> ${outFile} 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "rm /tmp/${package}"
}


deploy_opentk()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi

   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}
   
   line=$(cat ${metaFile} ${storageFile} ${clientFile} ${managementFile} | sort | uniq | grep "$1")
    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")
    echo "Installing FhGFS OpenTK Library on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}

    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg -i --force-overwrite"
     check_cmd="dpkg-query -s"
     pack_exten="deb"
     package=$(find ${packdir}/ -name fhgfsopentklib_*_${arch}.${pack_exten} | xargs basename)
     packname="fhgfsopentklib"
    else
     cmd="rpm -U --replacepkgs"
     check_cmd="rpm -q"
     pack_exten="rpm"
     package=$(find ${packdir}/ -name fhgfs_opentk_lib*.${arch}.${pack_exten} | xargs basename)
     packname="fhgfs_opentk_lib-${dist}"
    fi
    if [ "$package" = "" ]
    then
     error="No suitable OpenTK library package found for host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi 
    # transfer to target host
    scp -oStrictHostKeyChecking=no ${packdir}/${package} ${host}:/tmp/ >> ${outFile} 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} /tmp/${package}" >> ${outFile} 2>&1
#    ssh -qq -oStrictHostKeyChecking=no ${host} "${check_cmd} ${packname}" > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to install ${package} on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi 
    ssh -qq -oStrictHostKeyChecking=no ${host} "rm /tmp/${package}"
    # check if host needs ib => recompile opentk_lib
    OPTIONS=$(grep ${host} ${ib_hosts_file} 2>/dev/null | cut -f 2 -d ",")
    if [ "$OPTIONS" != "" ]
    then
      OPTIONS="FHGFS_OPENTK_IBVERBS=1 ${OPTIONS}"
    fi
    if [ ${arch} = ppc64 ]; then OPTIONS="${OPTIONS} USER_CXXFLAGS='-m64' USER_LDFLAGS='-m64'"; fi
    if [ "$OPTIONS" != "" ]
    then
     ssh -qq -oStrictHostKeyChecking=no ${host} "cd /opt/fhgfs/src/fhgfs_opentk_lib/build; make clean; make ${OPTIONS}" >> ${outFile} 2>&1
     if [ $? -ne 0 ] 
     then
#       cat ${tmpFile} >> ${logFile}
#       rm ${tmpFile}
       echo "" >> ${outFile}
       echo "Failed to compile OpenTK library on host ${host}." >> ${outFile}
       echo "The OpenTK library can still be used, but without native infiniband support." >> ${outFile}
#       echo $error >> $logFile
       echo "Options have been : ${OPTIONS}" >> $outFile
       echo ${host} >> ${failedFile}
       return 1
#     else
#       cat ${tmpFile} >> ${logFile}
#       rm ${tmpFile}
     fi
     ssh -qq -oStrictHostKeyChecking=no ${host} "cd /opt/fhgfs/src/fhgfs_opentk_lib/build; make install ${OPTIONS}" >> ${outFile} 2>&1 
     if [ $? -ne 0 ] 
     then
#       cat ${tmpFile} >> ${logFile}
#       rm ${tmpFile}
       echo "" >> ${outFile}
       echo "Failed to install OpenTK library on host ${host}." >> ${outFile}
       echo "The OpenTK library can still be used, but without native infiniband support." >> ${outFile}
#       echo $error >> $logFile
       echo "Options have been : ${OPTIONS}" >> $outFile
       echo ${host} >> ${failedFile}
       return 1
#     else
#       cat ${tmpFile} >> ${logFile}
#       rm ${tmpFile}
     fi
    fi
}

deploy_client()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi
   
   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}

   line=$(grep "$1" $clientFile)
  
    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")
    echo "Installing FhGFS Helper Daemon (dependency for client) on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg -i --force-overwrite"
     check_cmd="dpkg-query -s"
     pack_exten="deb"
     package=$(find ${packdir}/ -name fhgfshelperd_*_${arch}.${pack_exten} | xargs basename)
     packname="fhgfshelperd"
    else
     cmd="rpm -U --replacepkgs"
     check_cmd="rpm -q"
     pack_exten="rpm"
     package=$(find ${packdir}/ -name fhgfs_helperd-${dist}*.${arch}.${pack_exten} | xargs basename)
     packname="fhgfs_helperd-${dist}"
    fi
    # helperd
    if [ "$package" = "" ]
    then
     error="No suitable helper daemon package found for host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
    scp -oStrictHostKeyChecking=no ${packdir}/${package} ${host}:/tmp/ >> ${outFile} 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} /tmp/${package}" >> ${outFile} 2>&1
#    ssh -qq -oStrictHostKeyChecking=no ${host} "${check_cmd} ${packname}" > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to install ${package} on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "rm /tmp/${package}"
    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_helperd.conf.master ${host}:/etc/fhgfs/fhgfs_helperd.conf >> ${outFile} 2>&1
   
   echo "--------------------" >> ${outFile}

  # client_mod
    echo "Installing FhGFS Client Module on host ${host}" >> ${outFile}
    if [ "$dist" = debian ]
    then
#     cmd="dpkg -i --force-overwrite"
#     pack_exten="deb"
     package=$(find ${packdir}/ -name fhgfsclient_*_${arch}.${pack_exten} | xargs basename)
     packname="fhgfsclient"
    else
#     cmd="rpm -U --replacepkgs"
#     pack_exten="rpm"
     package=$(find ${packdir}/ -name fhgfs_client-${dist}*.${arch}.${pack_exten} | xargs basename)
     packname="fhgfs_client-${dist}"
    fi
    if [ "$package" = "" ]
    then
     error="No suitable client package found for host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
    # transfer to target host
    scp -oStrictHostKeyChecking=no ${packdir}/${package} ${host}:/tmp/ >> ${outFile} 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} /tmp/${package}" >> ${outFile} 2>&1
    # ssh -qq -oStrictHostKeyChecking=no ${host} "${check_cmd} ${packname}" > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to install ${package} on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "rm /tmp/${package}"
    # check for ib options and compile client
    OPTIONS=$(grep ${host} ${ib_hosts_file} | cut -f 2 -d ",")
    if [ "$OPTIONS" != "" ]
    then
      OPTIONS="FHGFS_OPENTK_IBVERBS=1 ${OPTIONS}"
    fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "cd /opt/fhgfs/src/client/fhgfs_client_module/build; make clean; make ${OPTIONS}" >> ${outFile} 2>&1
    if [ $? -ne 0 ]
     then
#       cat ${tmpFile} >> ${logFile}
#       rm ${tmpFile}
       echo "" >> ${outFile}
       error="Failed to compile client module on host ${host}"
       echo $error >> $outFile
       echo "Options have been : ${OPTIONS}" >> $outFile
       echo ${host} >> ${failedFile}
       return 1
#     else
#       cat ${tmpFile} >> ${logFile}
#       rm ${tmpFile}
     fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "cd /opt/fhgfs/src/client/fhgfs_client_module/build; make install ${OPTIONS}" >> ${outFile} 2>&1
    if [ $? -ne 0 ]
     then
#       cat ${tmpFile} >> ${logFile}
#       rm ${tmpFile}
       echo "" >> ${outFile}
       error="Failed to install client module on host ${host}"
       echo $error >> $outFile
       echo "Options have been : ${OPTIONS}" >> $outFile
       echo ${host} >> ${failedFile}
       return 1
#     else
#       cat ${tmpFile} >> ${outFile}
#       rm ${tmpFile}
     fi
    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_client.conf.master ${host}:/etc/fhgfs/fhgfs_client.conf >> ${outFile} 2>&1
    clientMount=$(cat ${confdir}/fhgfs_mounts.conf | cut -f 1 -d " ")
    if [ $clientMount != "" ]
    then
      ssh -qq -oStrictHostKeyChecking=no $host "mkdir -p ${clientMount}" >> ${outFile} 2>&1
    fi
    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_mounts.conf ${host}:/etc/fhgfs/ >> ${outFile} 2>&1
}

deploy_mgmtd()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi
   
   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}
   
    line=$(grep "$1" $managementFile)
    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")
    echo "Installing FhGFS Management Daemon on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg -i --force-overwrite"
     check_cmd="dpkg-query -s"
     pack_exten="deb"
     package=$(find ${packdir}/ -name fhgfsmgmtd_*_${arch}.${pack_exten} | xargs basename)
     packname="fhgfsmgmtd"
    else
     cmd="rpm -U --replacepkgs"
     check_cmd="rpm -q"
     pack_exten="rpm"
     package=$(find ${packdir}/ -name fhgfs_mgmtd-${dist}*.${arch}.${pack_exten} | xargs basename) 
     packname="fhgfs_mgmtd-${dist}"
    fi
    if [ "$package" = ""  ]
    then
     error="No suitable management daemon package found for host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}     
     return 1
    fi
    scp -oStrictHostKeyChecking=no ${packdir}/${package} ${host}:/tmp/ >> ${outFile} 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} /tmp/${package}" >> ${outFile} 2>&1
#    ssh -qq -oStrictHostKeyChecking=no ${host} "${check_cmd} ${packname}" > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to install ${package} on host ${host}"
     echo $error >> ${outFile}
     echo ${host} >> ${failedFile}
     return 1
    fi 
    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_mgmtd.conf.master ${host}:/etc/fhgfs/fhgfs_mgmtd.conf >> $outFile 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "rm /tmp/${package}"
}

if [ "$1" = "" ];then exit 1; fi
if [ "$2" = "" ];then exit 1; fi

if [ "$1" = "opentk" ]
then
  deploy_opentk $2 $3 $4
  exit $?
elif [ "$1" = "meta" ]
then
  deploy_meta $2 $3 $4
  exit $? 
elif [ "$1" = "storage" ]
then
  deploy_storage $2 $3 $4
  exit $? 
elif [ "$1" = "client" ]
then
  deploy_client $2 $3 $4
  exit $?
elif [ "$1" = "mgmtd" ]
then
  deploy_mgmtd $2 $3 $4
  exit $?  
else
  exit 1
fi
