#!/bin/bash

dirname=$(dirname $0)

. ${dirname}/fhgfs_setup.defaults


deploy_meta()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi
   
   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}
   
    host=$1
    echo "Deploying new configuration for Meta Server on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    
    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_meta.conf.master ${host}:/etc/fhgfs/fhgfs_meta.conf >> $outFile 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to deploy new meta server config on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
}

deploy_storage()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi

   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}

    host=$1
    echo "Deploying new configuration for Storage Server on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}

    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_storage.conf.master ${host}:/etc/fhgfs/fhgfs_storage.conf >> $outFile 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to deploy new storage server config on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
}

deploy_client()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi

   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}

    host=$1
    echo "Deploying new configuration for client on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}

    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_client.conf.master ${host}:/etc/fhgfs/fhgfs_client.conf >> $outFile 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to deploy new client config on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_mounts.conf ${host}:/etc/fhgfs/ >> ${outFile} 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to deploy new client config on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
}

deploy_mgmtd()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi

   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}

    host=$1
    echo "Deploying new configuration for Management Daemon on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}

    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_mgmtd.conf.master ${host}:/etc/fhgfs/fhgfs_mgmtd.conf >> $outFile 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to deploy new management daemon config on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
}

deploy_admon()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi

   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}

    host=$1
    echo "Deploying new configuration for admon on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}

    scp -oStrictHostKeyChecking=no ${confdir}/fhgfs_admon.conf.master ${host}:/etc/fhgfs/fhgfs_admon.conf >> $outFile 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to deploy new admon config on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
}





if [ "$1" = "" ];then exit 1; fi
if [ "$2" = "" ];then exit 1; fi

if [ "$1" = "opentk" ]
then
  deploy_opentk $2 $3 $4
  exit $?
elif [ "$1" = "meta" ]
then
  deploy_meta $2 $3 $4
  exit $? 
elif [ "$1" = "storage" ]
then
  deploy_storage $2 $3 $4
  exit $? 
elif [ "$1" = "client" ]
then
  deploy_client $2 $3 $4
  exit $?
elif [ "$1" = "mgmtd" ]
then
  deploy_mgmtd $2 $3 $4
  exit $?  
elif [ "$1" = "admon" ]
then
  deploy_admon $2 $3 $4
  exit $?
else
  exit 1
fi
