#!/bin/bash

dirname=$(dirname $0)

. ${dirname}/fhgfs_setup.defaults

IFS="
"

NUM=0
QUEUE=""
MAX_NPROC=4
COUNTER=0

function queue() {
        QUEUE="$QUEUE $1"
        NUM=$(($NUM+1))
}

function regeneratequeue() {
        OLDREQUEUE=$QUEUE
        QUEUE=""
        NUM=0
        for PID in $OLDREQUEUE
        do
                if [ -d /proc/$PID  ] ; then
                        QUEUE="$QUEUE $PID"
                        NUM=$(($NUM+1))
                fi
        done
}

function checkqueue() {
        OLDCHQUEUE=$QUEUE
        for PID in $OLDCHQUEUE
        do
                if [ ! -d /proc/$PID ] ; then
                        regeneratequeue # at least one PID has finished
                        break
                fi
        done
}




function stop_meta()
{
   IFS="
"
   failed=""
   REMOTECMD="/etc/init.d/fhgfs_meta stop"
   for line in $(cat ${metaFile}) 
   do
    host=$(echo ${line} | cut -f 1 -d ",")
    (ssh -qq -oStrictHostKeyChecking=no ${host} "${REMOTECMD}" > ${dirname}/tmp/fhgfs_setup.meta.${host}.tmp 2>&1; if [ $? -eq 0 ]; then rm ${dirname}/tmp/fhgfs_setup.meta.${host}.tmp; fi ) &
    PID=$!
    queue $PID

    while [ $NUM -ge $MAX_NPROC ]; do
            checkqueue
            sleep 0.4
    done

   done
   wait # wait for ssh -qq -oStrictHostKeyChecking=no jobs to finish before continuing
for f in $(find ${dirname}/tmp/ -name fhgfs_setup.meta.*.tmp)
do
 h=$(echo ${f} | cut -d "." -f 3)
 failed="${failed} ${h}"
 for line in $(cat ${f}); do echo "[${h},stop fhgfs_meta] ${line}" >> ${logFile}; done
done 
IFS=" "
if [ "${failed}" != "" ]
   then
     for host in ${failed}; do echo ${host}; done  
     rm ${dirname}/tmp/fhgfs_setup.meta.*.tmp > /dev/null 2>&1
     return 1
   fi    

}

function stop_storage()
{
   IFS="
"
   failed=""
   REMOTECMD="/etc/init.d/fhgfs_storage stop"
   for line in $(cat ${storageFile}) 
   do
    host=$(echo ${line} | cut -f 1 -d ",")
    (ssh -qq -oStrictHostKeyChecking=no ${host} "${REMOTECMD}" > ${dirname}/tmp/fhgfs_setup.storage.${host}.tmp 2>&1; if [ $? -eq 0 ]; then rm ${dirname}/tmp/fhgfs_setup.storage.${host}.tmp; fi ) &
    PID=$!
    queue $PID

    while [ $NUM -ge $MAX_NPROC ]; do
            checkqueue
            sleep 0.4
    done

   done
   wait # wait for ssh -qq -oStrictHostKeyChecking=no jobs to finish before continuing
for f in $(find ${dirname}/tmp/ -name fhgfs_setup.storage.*.tmp)
do
 h=$(echo ${f} | cut -d "." -f 3)
 failed="${failed} ${h}"
 for line in $(cat ${f}); do echo "[${h},stop fhgfs_storage] ${line}" >> ${logFile}; done
done 
IFS=" "
if [ "${failed}" != "" ]
   then
     for host in ${failed}; do echo ${host}; done
     rm ${dirname}/tmp/fhgfs_setup.storage.*.tmp > /dev/null 2>&1
     return 1
   fi    

}

function stop_mgmtd()
{
   IFS="
"
   failed=""
   REMOTECMD="/etc/init.d/fhgfs_mgmtd stop"
   for line in $(cat ${managementFile})
   do
    host=$(echo ${line} | cut -f 1 -d ",")
    (ssh -qq -oStrictHostKeyChecking=no ${host} "${REMOTECMD}" > ${dirname}/tmp/fhgfs_setup.mgmtd.${host}.tmp 2>&1; if [ $? -eq 0 ]; then rm ${dirname}/tmp/fhgfs_setup.mgmtd.${host}.tmp; fi ) &
    PID=$!
    queue $PID

    while [ $NUM -ge $MAX_NPROC ]; do
            checkqueue
            sleep 0.4
    done

   done
   wait # wait for ssh -qq -oStrictHostKeyChecking=no jobs to finish before continuing
for f in $(find ${dirname}/tmp/ -name fhgfs_setup.mgmtd.*.tmp)
do
 h=$(echo ${f} | cut -d "." -f 3)
 failed="${failed} ${h}"
 for line in $(cat ${f}); do echo "[${h},stop fhgfs_mgmtd] ${line}" >> ${logFile}; done
done
IFS=" "
if [ "${failed}" != "" ]
   then
     for host in ${failed}; do echo ${host}; done
     rm ${dirname}/tmp/fhgfs_setup.mgmtd.*.tmp > /dev/null 2>&1
     return 1
   fi

}

function stop_admon()
{
   IFS="
"
   failed=""
   line=$(cat ${admonFile})
   if [ "${line}" != "" ]
   then
     host=$(echo ${line} | cut -f 1 -d ",")
     ssh -qq -oStrictHostKeyChecking=no ${host} "/etc/init.d/fhgfs_admon stop" | tee -a ${tmpFile}
     if [ $? -ne 0 ]
     then
      failed="${failed}${host} "
     fi
   fi
   if [ "${failed}" != "" ]
   then 
     for line in $(cat $tmpFile); do echo "[${host},stop fhgfs_admon] ${line}" >> ${logFile}; done
     cat > $tmpFile <<EOF
Failed to start the FhGFS Monitoring System on the following hosts :

`for host in ${failed}; do echo ${host}; done`

Detailed information can be found in `echo ${logFile}`
EOF
     dia_textbox "Error Summary" $tmpFile
     rm $tmpFile > /dev/null 2>&1
   fi

}

function stop_client()
{
#  failed_helperd=""
#   failed_client=""
   IFS="
"
   for line in $(cat ${clientFile}) 
   do
    REMOTECMD="/etc/init.d/fhgfs_helperd stop"
    host=$(echo ${line} | cut -f 1 -d ",")
    (ssh -qq -oStrictHostKeyChecking=no ${host} "${REMOTECMD}" > ${dirname}/tmp/fhgfs_setup.helperd.${host}.tmp 2>&1; if [ $? -eq 0 ]; then rm ${dirname}/tmp/fhgfs_setup.helperd.${host}.tmp; fi ) &
    PID=$!
    queue $PID

    while [ $NUM -ge $MAX_NPROC ]
    do
            checkqueue
            sleep 0.4
    done
    
    REMOTECMD="/etc/init.d/fhgfs_client stop"
    (ssh -qq -oStrictHostKeyChecking=no ${host} "${REMOTECMD}" > ${dirname}/tmp/fhgfs_setup.client.${host}.tmp 2>&1; if [ $? -eq 0 ]; then rm ${dirname}/tmp/fhgfs_setup.client.${host}.tmp; fi ) &
      PID=$!
      queue $PID

      while [ $NUM -ge $MAX_NPROC ]; do
            checkqueue
            sleep 0.4
      done 
   done
   wait # wait for ssh -qq -oStrictHostKeyChecking=no jobs to finish before continuing
for f in $(find ${dirname}/tmp/ -name fhgfs_setup.helperd.*.tmp)
do
 h=$(echo ${f} | cut -d "." -f 3)
 failed_helperd="${failed_helperd} ${h}"
 for line in $(cat ${f}); do echo "[${h},stop fhgfs_helperd] ${line}" >> ${logFile}; done
done 

for f in $(find ${dirname}/tmp/ -name fhgfs_setup.client.*.tmp)
do
 echo $f
 h=$(echo ${f} | cut -d "." -f 3)
 failed_client="${failed_client} ${h}"
 for line in $(cat ${f}); do echo "[${h},stop fhgfs_client] ${line}" >> ${logFile}; done
done 


failed=0
IFS=" "
if [ "${failed_helperd}" != "" ]
   then

     for host in ${failed_helperd}; do echo ${host}; done  

     rm ${dirname}/tmp/fhgfs_setup.helperd.*.tmp
     failed=1
   fi    

if [ "${failed_client}" != "" ]
   then

       for host in ${failed_client}; do echo ${host}; done  

     rm ${dirname}/tmp/fhgfs_setup.client.*.tmp > /dev/null 2>&1
     failed=1
   fi  
if [ "$failed" == "1" ]
then
  return 1
fi
}

#start_client()
#{
#  failed_helperd=""
#   failed_client=""
#   for line in $(cat ${clientFile})
#   do
#    host=$(echo ${line} | cut -f 1 -d ",")
#    ssh -qq -oStrictHostKeyChecking=no ${host} "/etc/init.d/fhgfs_helperd start" | tee -a ${logFile}
#    if [ $? -ne 0 ]
#    then
#     failed_helperd="${failed_helperd}${host} "
#     # continue
#    fi
#    ssh -qq -oStrictHostKeyChecking=no ${host} "/etc/init.d/fhgfs_client start" | tee -a ${logFile}
#    if [ $? -ne 0 ]
#    then
#     failed_client="${failed_client}${host} "
#     #continue
#    fi
#   done
#  if [ "${failed_helperd}" != "" ]
#   then
#     cat > $tmpFile <<EOF
#Failed to start FhGFS Helper Daemon on the following hosts :
#
#`for host in ${failed_helperd}; do echo ${host}; done`
#
#Detailed information can be found in `echo ${logFile}`
#EOF
#     dia_msgbox $tmpFile
#     rm $tmpFile
#   fi
#
#   if [ "${failed_client}" != "" ]
#   then
#     cat > $tmpFile <<EOF
#Failed to start FhGFS Client on the following hosts :
#
#`for host in ${failed_client}; do echo ${host}; done`
#
#Detailed information can be found in `echo ${logFile}`
#EOF
#     dia_msgbox $tmpFile
#     rm $tmpFile
#   fi
#}

if [ "$1" = "" ];then exit 1; fi

if [ "$1" = "meta" ]
then
  stop_meta
  exit $?
elif [ "$1" = "storage" ]
then
  stop_storage
  exit $?
elif [ "$1" = "mgmtd" ]
then
  stop_mgmtd
  exit $?
elif [ "$1" = "client" ]
then
  stop_client
  exit $?
else
  exit 1
fi
