#!/bin/bash

dirname=$(dirname $0)

. ${dirname}/fhgfs_setup.defaults


uninstall_meta()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi
   
   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}
   
    line=$(grep "$1" $metaFile)
    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")
    echo "Un-Installing FhGFS Meta Server on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    
    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg --purge"
     packname="fhgfsmeta"
    else
     cmd="rpm -ve"
     packname="fhgfs_meta-${dist}"
    fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} ${packname}" >> ${outFile} 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to uninstall ${packname} on host ${host}"
     echo $error >> ${outFile}
     echo ${host} >> ${failedFile}
     return 1
    fi 
}

uninstall_storage()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi
   
   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}

    line=$(grep "$1" $storageFile)
    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")
    echo "Un-Installing FhGFS Storage Server on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg --purge"
     packname="fhgfsstorage"
    else
     cmd="rpm -ve"
     packname="fhgfs_storage-${dist}"
    fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} ${packname}" >> ${outFile} 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to uninstall ${packname} on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
}


uninstall_opentk()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi

   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}
   
   line=$(cat ${metaFile} ${storageFile} ${clientFile} ${admonFile} | sort | uniq | grep "$1")
    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")
    echo "Un-Installing FhGFS OpenTK Library on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}

    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg --purge"
     packname="fhgfsopentklib"
    else
     cmd="rpm -ve"
     packname="fhgfs_opentk_lib"
    fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "cd /opt/fhgfs/src/fhgfs_opentk_lib/build && make clean" >> /dev/null 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} ${packname}" >> ${outFile} 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to uninstall ${packname} on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi 
}

uninstall_client()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi
   
   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}

   line=$(grep "$1" $clientFile)

    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")


# client_mod
    echo "Un-Installing FhGFS Client Module on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    if [ "$dist" = debian ]
    then
     cmd="dpkg --purge"
     packname="fhgfsclient"
    else
     cmd="rpm -ve"
     packname="fhgfs_client-${dist}"
    fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "rm ${confdir}/fhgfs_mounts.conf /opt/fhgfs/client/*" >> ${outFile} 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "cd /opt/fhgfs/src/client/fhgfs_client_module/build && make clean && cd /opt/fhgfs/src/client/fhgfs_opentk_client_module/build && make clean" >> /dev/null 2>&1
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} ${packname}" >> ${outFile} 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to uninstall ${packname} on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi

   echo "" > ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}

    echo "Un-Installing FhGFS Helper Daemon (dependency for client) on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg --purge"
     packname="fhgfshelperd"
    else
     cmd="rpm -ve"
     packname="fhgfs_helperd-${dist}"
    fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} ${packname}" >> ${outFile} 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to uninstall ${packname} on host ${host}"
     echo $error >> $outFile
     echo ${host} >> ${failedFile}
     return 1
    fi
   
}

uninstall_mgmtd()
{
   if [ "$1" = "" ]
   then
     return 1
   fi
   if [ "$2" = "" ]
   then
     outFile=${logFile}
   else
     outFile=$2
   fi
   if [ "$3" = "" ]
   then
     failedFile="/dev/null"
   else
     failedFile=$3
   fi
   
   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}
   
    line=$(grep "$1" $managementFile)
    host=$(echo ${line} | cut -f 1 -d ",")
    arch=$(echo ${line} | cut -f 2 -d ",")
    dist=$(echo ${line} | cut -f 4 -d ",")
    echo "Un-Installing FhGFS Management Daemon on host ${host}" >> ${outFile}
    echo "--------------------" >> ${outFile}
    # depending on the arch we need a rpm or a deb package to install with rpm or dpkg
    # save the command and install it
    if [ "$dist" = debian ]
    then
     cmd="dpkg --purge"
     packname="fhgfsmgmtd"
    else
     cmd="rpm -ve"
     packname="fhgfs_mgmtd-${dist}"
    fi
    ssh -qq -oStrictHostKeyChecking=no ${host} "${cmd} ${packname}" >> ${outFile} 2>&1
    if [ $? -ne 0 ]
    then
     error="Failed to uninstall ${packname} on host ${host}"
     echo $error >> ${outFile}
     echo ${host} >> ${failedFile}
     return 1
    fi 
}

if [ "$1" = "" ];then exit 1; fi
if [ "$2" = "" ];then exit 1; fi

if [ "$1" = "opentk" ]
then
  uninstall_opentk $2 $3 $4
  exit $?
elif [ "$1" = "meta" ]
then
  uninstall_meta $2 $3 $4
  exit $? 
elif [ "$1" = "storage" ]
then
  uninstall_storage $2 $3 $4
  exit $? 
elif [ "$1" = "client" ]
then
  uninstall_client $2 $3 $4
  exit $?
elif [ "$1" = "mgmtd" ]
then
  uninstall_mgmtd $2 $3 $4
  exit $?  
else
  exit 1
fi
